app.addModule({beforeStart:function(app) {
app.addAssets([
	/*
		Change "density" parameter to change the number of randomly placed
		skins of each kind. Bigger values men less skins.
		
		Different speed values for testing:
		4 = no scates (2.5 tiles per second)
		5 = 2 scates
		6 = 4 scates
		7 = 6 scates or nyan-cat
		For each skin try to test it at least with "speed"=4 and "speed"=6
		
		One animation cycle takes ("animSpeed"/"speed") milliseconds. The
		default	value of "animSpeed" is 1200
		
		Rows and columns are counted from 0. The default order of rows is UP,
		RIGHT, DOWN, LEFT. If the order is different, specify "row" property:
		[n_up, n_right, n_down, n_left]
		where
			n_up = number of row that should be use for up animation,
			n_right = number of row that should be use for right animation, etc.
			
		"publiFrame" = [row number, column number] specifies what frame will be
		shown as a skin preview.
		
		For any number of frames except 3, the default number of column for
		standing (property "colStand") is 0 and all frames are played from 0th
		to the last. If "animCount" = 3, the "colStand"=1 and frames are played
		in this sequence: 1, 0, 1, 2, 1, 0, 1, 2, 1, ...
		If you are making a skin that has only 3 different frames (one for
		standing and two for walking), this is the best layout, since it
		conserves memory and you don't have to copy sprites while editing.
		
		EXAMPLES OF DIFFERENT SKIN CONFIGS ARE BELOW:
		(copy one of them and edit it to use for your skin)
	*/
	
	/*
		this example uses default values:
		4 columns,
		frameHeight = frameWidth 
		renderWidth = frameWidth (which means it's rendered in 1:1 scale, pixel-crisp)
		
		Its movement looks better with animSpeed=1600 (the default value is 1200,
		which is faster)
		
		What's bad with this skin (how not to create new skins):
		- it has too much unused space around sprites (especially horizontal)
		- its columns #0 and #2 are identical, so it could have been with only 3
		  columns to reduce memory consumption (see the 3-column example below)
	*/
	{	"name": "character_silver",
		"skin": {
			"frameWidth": 42,
			"animSpeed": 1600
		},
		"density": 128,
		"speed": 4
	},
	
	/*
		This example uses special 3 frames layout (see the explanation at the
		beginning of the file).
		
		Width and height are different.	renderWidth < frameWidth, so it is scaled
		down in the game.
		
		What's bad with this skin (how not to create new skins):
		- it has too much unused space around sprites
	*/
	{	"name": "hamster",
		"skin": {
			"animCount": 3,
			"frameWidth": 37,
			"frameHeight": 36,
			"renderWidth": 32
		},
		"density": 128,
		"speed": 4
	},
	
	
	/*
		This example has a non-standard order of rows (this order is usually
		used by RPG maker).Instead of UP, RIGHT, DOWN, LEFT, it is DOWN, LEFT,
		RIGHT, UP. The order in which rows sohuld be used is specified in the
		config by "row" parameter.
	*/
	{	"name": "blacklady",
		"skin": {
			"frameWidth": 32,
			"frameHeight": 48,
			"renderWidth": 28,
			"row": [3, 2, 0, 1]
		},
		"density": 128,
		"speed": 4
	},		
	
	/*
		this example uses default values:
		renderWidth = frameHeight = frameWidth = 32 (1:1 scale)
		animSpeed=1200

		It has 5 frames for walking up/down (specified by "animCount") and 8
		frames for walking left/right (specified by "animCountSide").
		
		In the same way, idle animation has both "idleAnimCount" and
		"idleAnimCountSide" (but we have no examples with "idleAnimCountSide"
		yet).
		
		It has no frames for moving left, so the skin has only 3 rows. For
		moving left, mirrored frames of right direction are used.
		
		What's bad with this skin (how not to do):
		- it has not a single free row of pixels above and below sprites, which
		causes graphical artifacts to appear
	*/
	{	"name": "bender32",
		"skin": {
			"frameWidth": 32,
			"animCount": 5,
			"animCountSide": 8,
			"animMirrorLeft": true,
			"animSpeed": 2400
		},
		"density": 128,
		"speed": 4
	},	
	
	/*
		This example has 6 animation frames and a special first colum for
		standing (7 frames total).
		
		"publicFrame" is used because 4-legged skins look better from the side.
		
		It may appear that this skin has too much free space, but in fact in some
		frames it has only 1 free pixel at the left, right or bottom side of the
		sprites. The only place where the size can be safely reduced is above the
		sprites.
	*/
	{	"name": "pony",
		"skin": {
			"publicFrame": [1, 0],
			"frameWidth": 55,
			"frameHeight": 54,
			"renderWidth": 45,
			"animStand": true,
			"animCount": 6,
			"animSpeed": 1800
		},
		"density": 128,
		"speed": 4
	},
	
	/*
		The image of acharater in this example is placed too low. Its legs
		would be rendered inside a horizontall wall if it try to move down
		against a wall.
		
		To fix it, we move image up in a config with "renderShiftY"
		parameter. If it is positive, it moves image down; if it's negative,
		it moves image up. In this example we've moved the image 6 pixels
		up.
	*/	
	{
		"name": "monte",
		"skin": {
			"publicFrame": [1, 0],
			"frameWidth": 64,
			"renderWidth": 48,
			"renderShiftY": -6,
			"animCount": 8,
			"animMirrorLeft": true
		},
		"density": 128,
		"speed": 4
	},	
	
	/*
		This example uses idle animation. By default it has the same number of
		frames as movement animation. Rows for idle animation are always stored
		after the rows of movement animation in the same order.
		
		Its idle animation speed is slower than it normal animation
		
		What's bad with this skin (how not to do):
		- it has not a single free row of pixels above and below sprites, which
		causes graphical artifacts to appear
	*/
	{	"name": "hulk",
		"skin": {
			"idleAnim": true,
			"idleAnimSpeed": 3000,
			"animSpeed": 1800
		},
		"density": 128,
		"speed": 4
	},
	
	/*
		This example uses a second kind of idle animation. It doesn't play all
		the time, but happens with random intervals. It is useful for actions
		such as blinking. "idlePauseMin" and "idlePauseMax" specify minimum and
		maximum random interval between animations. "idleAnimSpeed"=600 is a
		good small value for blinking.
		
		This skin has a different number of frames for idle animation than for
		moving, so "idleAnimCount" is specified.
		
		"publicFrame" is used because 4-legged skins look better from the side.
		
		What's bad with this skin (how not to create new skins):
		- it has too much unused space around sprites		
	*/
	{	"name": "White_Mouse",
		"skin": {
			"publicFrame" : [5, 0],
			"frameWidth": 48,
			"renderWidth": 40,
			"idleAnim": true,
			"idlePauseMin": 3000,
			"idlePauseMax": 10000,
			"idleAnimSpeed": 600,
			"idleAnimCount": 2
		},
		"density": 128,
		"speed": 4
	},
	
	/*
		This example uses a third kind of idle animation: it plays its usual
		animation of movement indefinitely - "alwaysRun" setting.
		
		If "idleAnimSpeed" is not specified (like in this example), its default
		value is (2 * "animSpeed"), so a character by default runs in place
		twice slower than when moving.
		
		It has a non-standard order of rows (this order is usually
		used by RPG maker).Instead of UP, RIGHT, DOWN, LEFT, it is DOWN, RIGHT,
		UP, left. The order in which rows sohuld be used is specified in the
		config by "row" parameter.
	*/	
	{	"name": "angel_of_dead4",
		"skin": {
			"frameWidth": 48,
			"alwaysRun": true,
			"row": [2, 1, 0, 3],
			"animSpeed": 1200
		},
		"density": 128,
		"speed": 4
	},
	
	/*
		This example demonstrates:
		
		1. Idle animation with a pause like the previous example.
		
		2. "idlePauseAfterMoving" parameter. It defines whether there should
		be a pause between the end of movement and the idle animation. The
		default value is true. It affects the skin only if it has
		"idlePauseMin" and "idlePauseMax" parameters.
		
		3. Repeating use of columns. It allows to create more complex animation
		sequences while including the image of each row only once and keeping
		the skin size small. The indexes of columns that should be displayed
		sequentially are specified in "col" and "idleCol" parameters (you can
		use one of them or both).
		
		In this example, "col" is used to play the same animation in once cycle
		forward and backward. "idleCol" is used to play the same animation
		sequence twice.		
	*/	
	{	"name": "slimetower",
		"skin": {
			"frameWidth": 32,
			"frameHeight": 40,
			"renderShiftY": -2,
			"col": [4,5,6,6,5,4,3,2,1,0,0,1,2,3,4],
			"animSpeed": 2000,
			"idleAnim": true,
			"idlePauseMin": 3000,
			"idlePauseMax": 10000,
			"idleAnimSpeed": 3000,
			"idleCol": [0,1,2,3,4,5,6,7,8,9,10,11,0,1,2,3,4,5,6,7,8,9,10,11],
			"idlePauseAfterMoving": false
		},
		"density": 128,
		"speed": 4
	}
	
])
}});