app.addModule({afterStart: function (app) {	

	var startX = 0, startY = 0, pressed = false, drag = false, dragSel = false;
	var TILE = 32;
	$("#builder").mousedown(function(e) {
		var x = e.pageX - e.target.offsetLeft - e.target.parentNode.offsetLeft- e.target.parentNode.parentNode.offsetLeft;
		var y = e.pageY - e.target.offsetTop - e.target.parentNode.offsetTop- e.target.parentNode.parentNode.offsetTop;
		app.builder.selectAt(x, y);
		app.builder.render();
		e.preventDefault();
		e.stopPropagation();
	});
	$("#screen").mousedown(function(e){
		pressed = true;
		drag = false;
		var renderer = app.renderer;
		var editor = app.game.editor;
		startX = e.pageX - e.target.offsetLeft;
		startY = e.pageY - e.target.offsetTop;
		dragSel = editor.selectionHasPoint(app.renderer.point(startX, startY));
		if (app.tool == 1)
			editor.pencilAt(renderer.point(startX, startY), 0);
		else if (app.tool == 2)
			editor.selectAt(renderer.point(startX, startY), 0);
		renderer.renderAll();
	});
	function doDrag(x, y) {
		var renderer = app.renderer;
		var editor = app.game.editor;
		if (!drag && (Math.abs(x-startX)>=5 || Math.abs(y-startY)>=5)) {
			drag = true;
		}
		if (drag) {
			var dx = x - startX, dy = y-startY;
			if (dragSel) {
				var tileDx = (dx/TILE) | 0, tileDy = (dy/TILE) | 0;
				if (tileDx!=0||tileDy!=0) {
					editor.moveSelection(tileDx, tileDy);
					startX += tileDx * TILE;
					startY += tileDy * TILE;
				}
			} else {
				startX = x;
				startY = y;
				renderer.moveCam(-dx, -dy);
			}
		}
		return drag;
	}
	
	$("#screen").mousemove(function(e) {
		if (!pressed) return;
		var renderer = app.renderer;
		var editor = app.game.editor;
		var x = e.pageX - e.target.offsetLeft;
		var y = e.pageY - e.target.offsetTop;
		if (app.tool == 0) {
			doDrag(x, y);
		} else if (app.tool == 1) {
			editor.pencilAt(renderer.point(x, y), 1);
		} else if (app.tool == 2) {
			editor.selectAt(renderer.point(x, y), 1);
		}
		renderer.renderAll();
	});
	$("#screen").mouseup(function(e){
		if (!pressed) return;
		var renderer = app.renderer;
		var editor = app.game.editor;
		pressed = false;
		var x = e.pageX - e.target.offsetLeft;
		var y = e.pageY - e.target.offsetTop;
		if (app.tool == 0) {
			if (!doDrag(x, y))
				editor.cursorAt(renderer.point(x,y));
		} else if (app.tool == 1) {
			editor.pencilAt(renderer.point(x, y), 2);
		} else if (app.tool == 2) {
			editor.selectAt(renderer.point(x, y), 2);
		}
		renderer.renderAll();
		renderer.roundCam();
	});
	var keyUp = false, keyDown = false, keyLeft = false, keyRight = false, 
		keySpace = false, keyEnter = false, redraw = false;
		
	var tick = 1000/60;
	var TILE_BITS = 5;
	setInterval(function() {
		var renderer = app.renderer;
		var game = app.game;
		var p = game.editor.entity;
		if (p==null || !p.type.animSpeed) return;
		var skin = p.type;
		
		var isMoving = keyUp || keyDown || keyLeft || keyRight || keySpace || keyEnter;
		var isPlayingIdleAnim = !isMoving && skin.idleAnim;
		if (!isMoving && !isPlayingIdleAnim && !skin.alwaysRun && !redraw) return;
		
		// preprocess entity
		p.step = p.step || 0;
		skin.hasIdleAnimPause = typeof skin.idlePauseMin != "undefined" && typeof skin.idlePauseMax != "undefined";
		if (typeof p.idleAnimPhase == "undefined")
			p.idleAnimPhase = "none";
		
		if (isMoving || skin.alwaysRun) {
			p.sy = p.sy % skin.framesY; // switch to normal rows after idleAnim
			p.idleAnimPhase = "none";
			game.obj[p.dy >> TILE_BITS][p.dx >> TILE_BITS] = null;
			var d = skin.speed * 2 * tick/100;
			if (keyUp) {
				p.sy = skin.row[0];
				p.dy = Math.max(1, p.dy-d);
				p.mirror = false;
			} else if (keyDown) {
				p.sy = skin.row[2];
				p.dy = Math.min((game.map.height<<TILE_BITS) - 1, p.dy+d);
				p.mirror = false;
			}else if (keyLeft) {
				if (skin.animMirrorLeft) {
					p.sy = skin.row[1]
					p.mirror = true;
				} else {
					p.sy = skin.row[3];
					p.mirror = false;
				}
				p.dx = Math.max(0, p.dx-d);
			} else if (keyRight) {
				p.sy = skin.row[1];
				p.dx = Math.min((game.map.width<<TILE_BITS) - 1, p.dx+d);
				p.mirror = false;
			}
			var d1 = isMoving
				? skin.speed * tick / skin.animSpeed
				: 4 * tick / skin.idleAnimSpeed;
			if (skin.framesY==2 && p.sy>=2) {
				p.step -= d1;
				p.sy -= 2;
			}else
				p.step += d1;
			var oldsx = p.sx;
			var animCount = (p.sy == skin.row[1] || p.sy == skin.row[3]) ?
					skin.animCountSide : skin.animCount;
			p.sx = skin.animStand
				? skin.col[1 + (p.step * (animCount - 1) | 0) % (animCount - 1)]
				: skin.col[(p.step * animCount | 0) % animCount];
			if(keyEnter && p.sx==0 && oldsx!==0)
				keyEnter = false;
			redraw = true;
			game.obj[p.dy >> TILE_BITS][p.dx >> TILE_BITS] = p;			
		}else if (isPlayingIdleAnim) {
			p.sy = p.sy % skin.framesY + skin.framesY; // switch to idleAnim rows after normal
			
			p.dx = p.dx | 0;
			p.dy = p.dy | 0;
			if (p.idleAnimPhase == "none") {
				p.step = 0;
				p.idleAnimPhase = "playing";
			}
			if (p.idleAnimPhase == "pause") {
				if (new Date().getTime() >= p.idlePauseEndsOn) {
					p.idleAnimPhase = "playing";
				}
			}
			if (p.idleAnimPhase == "playing") {
				p.step += 4 * tick / skin.idleAnimSpeed;
				var animCount = (p.sy == skin.row[1] || p.sy == skin.row[3]) ?
					skin.idleAnimCountSide : skin.idleAnimCount;				
				p.sx = skin.idleCol[ (p.step * animCount | 0) % animCount ];
				if (skin.hasIdleAnimPause && p.step >= 1) {
					p.step = 0;
					p.idlePauseEndsOn = new Date().getTime() +
						Math.random()*(skin.idlePauseMax - skin.idlePauseMin) +
						skin.idlePauseMin;
					p.idleAnimPhase = "pause";
				}
			}			
		}else { // just standing
			p.dx = p.dx | 0;
			p.dy = p.dy | 0;
			p.sx = skin.colStand || 0;
			p.step = 0;
			redraw = false;
		}

		renderer.renderAll();
	},tick);
	$(document).keydown(function(event){
		switch (event.which) {
			case 39:
			case 68:
				keyRight = true;
				break;
			case 38:
			case 87:
				keyUp = true;
				break;
			case 37:
			case 65:
				keyLeft = true;
				break;
			case 40:
			case 83:
				keyDown = true;
				break;
			case 13:
				keyEnter = true;
				break;						
			case 32:
				keySpace = true;
				break;						
		}
	});
	$(document).keyup(function(event){
		switch (event.which) {
			case 39:
			case 68:
				keyRight = false;
				break;
			case 38:
			case 87:
				keyUp = false;
				break;
			case 37:
			case 65:
				keyLeft = false;
				break;
			case 40:
			case 83:
				keyDown = false;
				break;
			case 13:
				//keyEnter = false;
				break;						
			case 32:
				keySpace = false;
				break;					
		}
	});
}})