app.addModule({afterStart: function (app) {	

	var startX = 0, startY = 0, pressed = false, drag = false, dragSel = false;
	var lastSkin = null;
	var TILE = 32;
	$("#builder").mousedown(function(e) {
		var x = e.pageX - e.target.offsetLeft - e.target.parentNode.offsetLeft- e.target.parentNode.parentNode.offsetLeft;
		var y = e.pageY - e.target.offsetTop - e.target.parentNode.offsetTop- e.target.parentNode.parentNode.offsetTop;
		app.builder.selectAt(x, y);
		app.builder.render();
		e.preventDefault();
		e.stopPropagation();
	});
	$("#screen").mousedown(function(e){
		pressed = true;
		drag = false;
		var renderer = app.renderer;
		var editor = app.game.editor;
		startX = e.pageX - e.target.offsetLeft;
		startY = e.pageY - e.target.offsetTop;
		dragSel = editor.selectionHasPoint(app.renderer.point(startX, startY));
		if (app.tool == 1)
			editor.pencilAt(renderer.point(startX, startY), 0);
		else if (app.tool == 2)
			editor.selectAt(renderer.point(startX, startY), 0);
		renderer.renderAll();
	});
	function doDrag(x, y) {
		var renderer = app.renderer;
		var editor = app.game.editor;
		if (!drag && (Math.abs(x-startX)>=5 || Math.abs(y-startY)>=5)) {
			drag = true;
		}
		if (drag) {
			var dx = x - startX, dy = y-startY;
			if (dragSel) {
				var tileDx = (dx/TILE) | 0, tileDy = (dy/TILE) | 0;
				if (tileDx!=0||tileDy!=0) {
					editor.moveSelection(tileDx, tileDy);
					startX += tileDx * TILE;
					startY += tileDy * TILE;
				}
			} else {
				startX = x;
				startY = y;
				renderer.moveCam(-dx, -dy);
			}
		}
		return drag;
	}
	
	$("#screen").mousemove(function(e) {
		if (!pressed) return;
		var renderer = app.renderer;
		var editor = app.game.editor;
		var x = e.pageX - e.target.offsetLeft;
		var y = e.pageY - e.target.offsetTop;
		if (app.tool == 0) {
			doDrag(x, y);
		} else if (app.tool == 1) {
			editor.pencilAt(renderer.point(x, y), 1);
		} else if (app.tool == 2) {
			editor.selectAt(renderer.point(x, y), 1);
		}
		renderer.renderAll();
	});
	$("#screen").mouseup(function(e){
		if (!pressed) return;
		var renderer = app.renderer;
		var editor = app.game.editor;
		pressed = false;
		var x = e.pageX - e.target.offsetLeft;
		var y = e.pageY - e.target.offsetTop;
		if (app.tool == 0) {
			if (!doDrag(x, y))
				editor.cursorAt(renderer.point(x,y));
		} else if (app.tool == 1) {
			editor.pencilAt(renderer.point(x, y), 2);
		} else if (app.tool == 2) {
			editor.selectAt(renderer.point(x, y), 2);
		}
		renderer.renderAll();
		renderer.roundCam();
	});
	var keyUp = false, keyDown = false, keyLeft = false, keyRight = false, 
		keySpace = false, keyEnter = false, redraw = false, lastArrow = "";
		
	var tick = 1000/60;
	var TILE_BITS = 5;
	setInterval(function() {
		var renderer = app.renderer;
		var game = app.game;
		var p = game.editor.entity;
		if (p==null || !p.type.animSpeed) return;
		var skin = p.type;
		
		// preprocess entity
		p.step = p.step || 0;
		skin.hasIdleAnimPause = typeof skin.idlePauseMin != "undefined" && typeof skin.idlePauseMax != "undefined";
		if (typeof p.idleAnimPhase == "undefined")
			p.idleAnimPhase = "none";
			
		function setIdlePause() {
			p.step = 0;
			p.idlePauseEndsOn = new Date().getTime() +
				Math.random()*(skin.idlePauseMax - skin.idlePauseMin) +
				skin.idlePauseMin;
			p.idleAnimPhase = "pause";
		}
			
		var curMapY = p.dy >> TILE_BITS;
		var curMapX = p.dx >> TILE_BITS;
		function nextCellFree(dx, dy) {
			tileId = app.map.getTile(
				Math.min(game.map.width-1, Math.max(0, curMapX+dx)),
				Math.min(game.map.height-1, Math.max(0, curMapY+dy))
			);
			tile = app.conf.tiles[tileId];
			return tile.type == "floor" || tile.type == "hideout" || tile.type == "building";
		}
		var minY = nextCellFree(0, -1) ? 0 : (curMapY << TILE_BITS) + 11;
		var maxY = nextCellFree(0, 1) ? (game.map.height<<TILE_BITS) - 1
				: (curMapY << TILE_BITS) + TILE_BITS + 14;
		var minX = nextCellFree(-1, 0) ? 0 : (curMapX << TILE_BITS) + 13;
		var maxX = nextCellFree(1, 0) ? (game.map.width<<TILE_BITS) - 1
				: (curMapX << TILE_BITS) + TILE_BITS + 14;
		var kUp    = p.dy != minY && keyUp;
		var kDown  = p.dy != maxY && keyDown;
		var kLeft  = p.dx != minX && keyLeft;
		var kRight = p.dx != maxX && keyRight;
		
		var isMoving = kUp || kDown || kLeft || kRight || keySpace || keyEnter;
		var isPlayingIdleAnim = !isMoving && skin.idleAnim;
		if (!isMoving && !isPlayingIdleAnim && !skin.alwaysRun && !redraw) return;
		
		if (isMoving || skin.alwaysRun) {
			p.sy = p.sy % skin.framesY; // switch to normal rows after idleAnim
			p.idleAnimPhase = "none";

			game.obj[curMapY][curMapX] = null;
			var d = skin.speed * 2 * tick/100;

			if (kUp && !kDown) {
				if (lastArrow == "u" || lastArrow == "")
					p.sy = skin.row[0];
				p.dy -= d;
				p.mirror = false;
			} else if (kDown) {
				if (lastArrow == "d" || lastArrow == "")
					p.sy = skin.row[2];
				p.dy += d;
				p.mirror = false;
			}
			if (kLeft && !kRight) {
				if (skin.animMirrorLeft) {
					if (lastArrow == "l" || lastArrow == "")
						p.sy = skin.row[1]
					p.mirror = true;
				} else {
					if (lastArrow == "l" || lastArrow == "")
						p.sy = skin.row[3];
					p.mirror = false;
				}
				p.dx -= d;
			} else if (kRight) {
				if (lastArrow == "r" || lastArrow == "")
					p.sy = skin.row[1];
				p.dx += d;
				p.mirror = false;
			}
			p.dy = Math.max(p.dy, minY);
			p.dy = Math.min(p.dy, maxY);
			p.dx = Math.max(p.dx, minX);
			p.dx = Math.min(p.dx, maxX);
			var d1 = isMoving
				? skin.speed * tick / skin.animSpeed
				: 4 * tick / skin.idleAnimSpeed;
			if (skin.framesY==2 && p.sy>=2) {
				p.step -= d1;
				p.sy -= 2;
			}else
				p.step += d1;
			var oldsx = p.sx;
			var animCount = (p.sy == skin.row[1] || p.sy == skin.row[3]) ?
					skin.animCountSide : skin.animCount;
			var acMax = Math.max(skin.animCount, skin.animCountSide);
			p.sx = skin.col[ (skin.animStand ? 1 : 0) + (p.step * acMax | 0) % animCount];
			if(keyEnter && p.sx==0 && oldsx!==0)
				keyEnter = false;
			redraw = true;
			game.obj[p.dy >> TILE_BITS][p.dx >> TILE_BITS] = p;			
		}else if (isPlayingIdleAnim) {
			p.sy = p.sy % skin.framesY + skin.framesY; // switch to idleAnim rows after normal
			
			p.dx = p.dx | 0;
			p.dy = p.dy | 0;
			if (p.idleAnimPhase == "none") {
				if (skin.hasIdleAnimPause && skin.idlePauseAfterMoving && lastSkin == p) {
					setIdlePause();
				}else {
					p.step = 0;
					p.idleAnimPhase = "playing";
				}
			}
			if (p.idleAnimPhase == "pause") {
				if (new Date().getTime() >= p.idlePauseEndsOn) {
					p.idleAnimPhase = "playing";
				}
			}
			if (p.idleAnimPhase == "playing") {
				p.step += 4 * tick / skin.idleAnimSpeed;
				var animCount = (p.sy == skin.row[1] || p.sy == skin.row[3]) ?
					skin.idleAnimCountSide : skin.idleAnimCount;
				if (skin.hasIdleAnimPause && p.step >= 1)
					setIdlePause();
			}
			var acMax = Math.max(skin.idleAnimCount, skin.idleAnimCountSide);
			p.sx = skin.idleCol[ (p.step * acMax | 0) % acMax ];
		}else { // just standing
			p.dx = p.dx | 0;
			p.dy = p.dy | 0;
			p.sx = skin.colStand || 0;
			p.step = 0;
			redraw = false;
		}
		lastSkin = p;

		renderer.renderAll();
	},tick);
	$(document).keydown(function(event){
		switch (event.which) {
			case 39:
			case 68:
				keyRight = true;
				lastArrow = "r";
				break;
			case 38:
			case 87:
				keyUp = true;
				lastArrow = "u";
				break;
			case 37:
			case 65:
				keyLeft = true;
				lastArrow = "l";
				break;
			case 40:
			case 83:
				keyDown = true;
				lastArrow = "d";
				break;
			case 13:
				keyEnter = true;
				break;						
			case 32:
				keySpace = true;
				break;						
		}
	});
	$(document).keyup(function(event){
		switch (event.which) {
			case 39:
			case 68:
				keyRight = false;
				lastArrow = "";
				break;
			case 38:
			case 87:
				keyUp = false;
				lastArrow = "";
				break;
			case 37:
			case 65:
				keyLeft = false;
				lastArrow = "";
				break;
			case 40:
			case 83:
				keyDown = false;
				lastArrow = "";
				break;
			case 13:
				//keyEnter = false;
				break;						
			case 32:
				keySpace = false;
				break;					
		}
	});
}})